# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stratum1/p4_role_config.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dstratum1/p4_role_config.proto\x12\x07stratum\"\xe9\x01\n\x0cP4RoleConfig\x12\x18\n\x10\x65xclusive_p4_ids\x18\x01 \x03(\r\x12\x15\n\rshared_p4_ids\x18\x02 \x03(\r\x12<\n\x10packet_in_filter\x18\x03 \x01(\x0b\x32\".stratum.P4RoleConfig.PacketFilter\x12\x1b\n\x13receives_packet_ins\x18\x04 \x01(\x08\x12\x19\n\x11\x63\x61n_push_pipeline\x18\x05 \x01(\x08\x1a\x32\n\x0cPacketFilter\x12\x13\n\x0bmetadata_id\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\x0c\x42\x03\x80\x01\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stratum1.p4_role_config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\200\001\000'
  _globals['_P4ROLECONFIG']._serialized_start=43
  _globals['_P4ROLECONFIG']._serialized_end=276
  _globals['_P4ROLECONFIG_PACKETFILTER']._serialized_start=226
  _globals['_P4ROLECONFIG_PACKETFILTER']._serialized_end=276
# @@protoc_insertion_point(module_scope)
